@@ -96,12 +96,17 @@ MOBI_RET mobi_build_opf_guide(OPF *opf, const MOBIRawml *rawml) {
         debug_print("%s\n", "Initialization failed");
         return MOBI_INIT_FAILED;
     }
-    size_t i = 0, j = 0;
+    size_t i = 0;
+    size_t j = 0;
     MOBI_RET ret;
     size_t count = rawml->guide->entries_count;
     if (count == 0) {
         return MOBI_SUCCESS;
     }
+    if (rawml->frag == NULL) {
+        debug_print("%s\n", "Missing frag part");
+        return MOBI_DATA_CORRUPT;
+    }
     opf->guide = malloc(sizeof(OPFguide));
     if (opf->guide == NULL) {
         debug_print("%s\n", "Memory allocation failed");
@@ -151,7 +156,7 @@ MOBI_RET mobi_build_opf_guide(OPF *opf, const MOBIRawml *rawml) {
             /* FIXME: I need some examples which use other tags */
             //mobi_get_indxentry_tagvalue(&frag_number, guide_entry, INDX_TAG_FRAG_FILE_NR);
         }
-        if (frag_number > rawml->frag->entries_count) {
+        if (frag_number >= rawml->frag->entries_count) {
             debug_print("Wrong frag entry index (%i)\n", frag_number);
             free(ref_title);
             i++;
@@ -198,7 +203,7 @@ MOBI_RET mobi_build_opf_guide(OPF *opf, const MOBIRawml *rawml) {
         }
         debug_print("<reference type=\"%s\" title=\"%s\" href=\"part%05u.html\" />", ref_type, ref_title, file_number);
         char href[FILENAME_MAX + 1];
-        snprintf(href, FILENAME_MAX, "part%05u.html", file_number);
+        snprintf(href, sizeof(href), "part%05u.html", file_number);
         char *ref_href = strdup(href);
         reference[j] = calloc(1, sizeof(OPFreference));
         *reference[j] = (OPFreference) { ref_type, ref_title, ref_href };
@@ -672,9 +677,9 @@ MOBI_RET mobi_build_ncx(MOBIRawml *rawml, const OPF *opf) {
                 mobi_free_ncx(ncx, i);
                 return ret;
             }
-            if ((first_child != MOBI_NOTSET && first_child > rawml->ncx->entries_count) ||
-                (last_child != MOBI_NOTSET && last_child > rawml->ncx->entries_count) ||
-                (parent != MOBI_NOTSET && parent > rawml->ncx->entries_count)) {
+            if ((first_child != MOBI_NOTSET && first_child >= rawml->ncx->entries_count) ||
+                (last_child != MOBI_NOTSET && last_child >= rawml->ncx->entries_count) ||
+                (parent != MOBI_NOTSET && parent >= rawml->ncx->entries_count)) {
                 free(text);
                 free(target);
                 mobi_free_ncx(ncx, i);
@@ -1151,32 +1156,38 @@ MOBI_RET mobi_build_opf_metadata(OPF *opf,  const MOBIData *m, const MOBIRawml *
     if (mobi_is_dictionary(m)) {
         if (opf->metadata->x_meta->dictionary_in_lang == NULL) {
             if (m->mh && m->mh->dict_input_lang) {
-                opf->metadata->x_meta->dictionary_in_lang = calloc(OPF_META_MAX_TAGS, sizeof(char*));
-                if (opf->metadata->x_meta->dictionary_in_lang == NULL) {
-                    debug_print("%s\n", "Memory allocation failed");
-                    return MOBI_MALLOC_FAILED;
-                }
                 uint32_t dict_lang_in = *m->mh->dict_input_lang;
-                opf->metadata->x_meta->dictionary_in_lang[0] = strdup(mobi_get_locale_string(dict_lang_in));
+                const char *lang = mobi_get_locale_string(dict_lang_in);
+                if (lang) {
+                    opf->metadata->x_meta->dictionary_in_lang = calloc(OPF_META_MAX_TAGS, sizeof(char*));
+                    if (opf->metadata->x_meta->dictionary_in_lang == NULL) {
+                        debug_print("%s\n", "Memory allocation failed");
+                        return MOBI_MALLOC_FAILED;
+                    }
+                    opf->metadata->x_meta->dictionary_in_lang[0] = strdup(lang);
+                }
             }
         }
         if (opf->metadata->x_meta->dictionary_out_lang == NULL) {
             if (m->mh && m->mh->dict_output_lang) {
-                opf->metadata->x_meta->dictionary_out_lang = calloc(OPF_META_MAX_TAGS, sizeof(char*));
-                if (opf->metadata->x_meta->dictionary_out_lang == NULL) {
-                    debug_print("%s\n", "Memory allocation failed");
-                    return MOBI_MALLOC_FAILED;
+                uint32_t dict_lang_out = *m->mh->dict_output_lang;
+                const char *lang = mobi_get_locale_string(dict_lang_out);
+                if (lang) {
+                    opf->metadata->x_meta->dictionary_out_lang = calloc(OPF_META_MAX_TAGS, sizeof(char*));
+                    if (opf->metadata->x_meta->dictionary_out_lang == NULL) {
+                        debug_print("%s\n", "Memory allocation failed");
+                        return MOBI_MALLOC_FAILED;
+                    }
+                    opf->metadata->x_meta->dictionary_out_lang[0] = strdup(lang);
                 }
-                uint32_t dict_lang_in = *m->mh->dict_output_lang;
-                opf->metadata->x_meta->dictionary_out_lang[0] = strdup(mobi_get_locale_string(dict_lang_in));
             }
         }
-        if (rawml->orth->orth_index_name) {
+        if (rawml->orth && rawml->orth->orth_index_name) {
             opf->metadata->x_meta->default_lookup_index = calloc(OPF_META_MAX_TAGS, sizeof(char*));
-			if (opf->metadata->x_meta->default_lookup_index == NULL) {
-				debug_print("%s\n", "Memory allocation failed");
-				return MOBI_MALLOC_FAILED;
-			}
+            if (opf->metadata->x_meta->default_lookup_index == NULL) {
+                debug_print("%s\n", "Memory allocation failed");
+                return MOBI_MALLOC_FAILED;
+            }
             opf->metadata->x_meta->default_lookup_index[0] = strdup(rawml->orth->orth_index_name);
         }
     }
